#include "Actions.h"

TimeAction::TimeAction(unsigned long activeTime, void (*func)(float))  :
	_isDone(false),
	_ellapsedTime(0) {
  _activeTime = activeTime; 
  this->action = func;
}

void TimeAction::update() {
	_ellapsedTime += Timer::getInstance()->delta();
}

void TimeAction::reset() {
	_ellapsedTime = 0;
	_isDone = false;
}

boolean TimeAction::isDone() {
	return _isDone;
}

OneTimeAction::OneTimeAction(unsigned long workTime, void (*func)(float)) :
  TimeAction(workTime, func) { }

void OneTimeAction::update() {
	TimeAction::update();
    if (_ellapsedTime >= _activeTime) {
    	_isDone = true;
    	if (action != NULL) {
        	action(Timer::getInstance()->delta()); 
    	}
    }
}

MultiTimeAction::MultiTimeAction(int activeCount, unsigned long workTime, void (*func)(float)) :
  TimeAction(workTime, func),
  _currentCount(0){ 
	_activeCount = activeCount;
}

void MultiTimeAction::update() {
	TimeAction::update();
    if (_ellapsedTime >= _activeTime) {
    	if (action != NULL) {
        	action(Timer::getInstance()->delta()); 
    	}
		_currentCount++;
		_ellapsedTime -= _activeTime;
		if (_currentCount == _activeCount) {
			_isDone = true;
		}
    }
}

void MultiTimeAction::reset() {
	TimeAction::reset();
	_currentCount = 0;
}

ProgressAction::ProgressAction(unsigned long workTime, void (*func)(float)) :
  TimeAction(workTime, func) { }

void ProgressAction::update() {
     if (!_isDone) {
     	TimeAction::update();
     	if (_ellapsedTime < _activeTime) {
        	if (action != NULL) {
        		action((float)_ellapsedTime / _activeTime); 
         	}
       	} else {
         	_isDone = true;
         	if (action != NULL) {
           		action(1.0f); 
         	}
       	}
	}
}

RepeatForeverAction::RepeatForeverAction(unsigned long workTime, void (*func)(float)) :
  TimeAction(workTime, func) { }

void RepeatForeverAction::update() {
     TimeAction::update();
     if (_ellapsedTime >= _activeTime) {
       	reset();
       	if (action != NULL) {
         	action(Timer::getInstance()->delta()); 
       	}
 	} 
}

SequenceAction::SequenceAction() :
  TimeAction(0UL, NULL),
  _head(NULL){
}

void SequenceAction::update() {
  	if (!_isDone) {
    	TimeAction::update();
    	if (!_head->value->isDone()) {
      		_head->value->update();
    	} else {
			LinkedNode<TimeAction*> *tmp = _head;
			_head = _head->next;
			tmp = NULL; // for memory cleanup
      		if (_head == NULL) {
        		_isDone = true;
      		}
    	}
  	}
}

SequenceAction* SequenceAction::add(TimeAction* action) {
	if (_head == NULL) {
		_head = new LinkedNode<TimeAction*>();
		_head->value = action;
		_head->next = NULL;
	} else {
		LinkedNode<TimeAction*> *current = _head;
		while(current->next != NULL) {
			current = current->next;
		}
		current->next = new LinkedNode<TimeAction*>();
		current->next->value = action;
		current->next->next = NULL;
	}
	return this;
}


ParallelAction::ParallelAction() :
  TimeAction(0UL, NULL),
  _head(NULL){
}

void ParallelAction::update() {
  	if (!_isDone) {
    	TimeAction::update();
		LinkedNode<TimeAction*> *current = _head;
		bool isAllDone = true;
		while (current != NULL) {
			current->value->update();
			if (!current->value->isDone()) {
				isAllDone = false;
			}
			current = current->next;
		}
		_isDone = isAllDone;
  	}
}

ParallelAction* ParallelAction::add(TimeAction* action) {
	if (_head == NULL) {
		_head = new LinkedNode<TimeAction*>();
		_head->value = action;
		_head->next = NULL;
	} else {
		LinkedNode<TimeAction*> *current = _head;
		while(current->next != NULL) {
			current = current->next;
		}
		current->next = new LinkedNode<TimeAction*>();
		current->next->value = action;
		current->next->next = NULL;
	}
	return this;
}