#ifndef __ACTIONS_H__
#define __ACTIONS_H__

#include "Timer.h"
#include "Array.h"

class TimeAction {
  protected:
    unsigned long _ellapsedTime;
    unsigned long _activeTime;
    boolean _isDone;
    void (*action)(float);
  public:
	TimeAction(unsigned long activeTime, void (*func)(float));
    
	virtual void update();
    
	virtual void reset();
    
	boolean isDone();
};

class OneTimeAction : public TimeAction {
  public:
	OneTimeAction(unsigned long workTime, void (*func)(float));
      
	void update();
};

class MultiTimeAction : public TimeAction {
	protected:
		int _currentCount;
		int _activeCount;
	public:
		MultiTimeAction(int activeCount, unsigned long workTime, void (*func)(float));
      
		void update();
		
		virtual void reset();
};


class ProgressAction : public TimeAction {
  public:
	ProgressAction(unsigned long workTime, void (*func)(float));
      
	void update();
};

class RepeatForeverAction : public TimeAction {
   public:
	RepeatForeverAction(unsigned long workTime, void (*func)(float));
      
	void update();
};

class SequenceAction : public TimeAction {
  protected:
    LinkedNode<TimeAction*> *_head;
  public:
	SequenceAction();
    
	void update();
	
	SequenceAction* add(TimeAction* action);
};


class ParallelAction : public TimeAction {
  protected:
    LinkedNode<TimeAction*> *_head;
  public:
	ParallelAction();
    
	void update();
	
	ParallelAction* add(TimeAction* action);
};

#endif